from typing import Any, TypeAlias, Union, Literal, List, Dict
from io import BytesIO
from PIL.Image import Image as _Image

__all__: list[str] = [
    "PILImage",
    "Prompt",
    "Model",
    "Seed",
    "Private",
    "File",
    "Referrer",
    "Stream",
    "Filename",
    "ImageData",
    "Infinity",
    "Time",
    "Payload",
    "Params",
    "EmptyParams",
    "StreamData",
    "ErrorMessage",
    "Args",
    "Kwargs",
    "Negative",
    "Width",
    "Height",
    "NoLogo",
    "Enhance",
    "Safe",
    "Save",
    "System",
    "Contextual",
    "Messages",
    "Role",
    "Message",
    "JsonMode",
    "ReasoningEffort",
    "Tools",
    "ToolChoice",
    "Voice",
    "UseOpenAIEndpoint",
    "Images",
    "ImageFormat",
    "Output",
    "FeedType",
    "MaxData"
]

# All
Prompt: TypeAlias = str
Model: TypeAlias = str
Seed: TypeAlias = Union[int, str]
Private: TypeAlias = bool
File: TypeAlias = str
Referrer: TypeAlias = str
Stream: TypeAlias = bool
Filename: TypeAlias = str
ImageData: TypeAlias = Union[str, BytesIO]
PILImage: TypeAlias = _Image

Infinity: TypeAlias = None
Time: TypeAlias = float

Payload: TypeAlias = dict[str, Any]
Params: TypeAlias = dict[str, Any]
EmptyParams: TypeAlias = dict

StreamData: TypeAlias = str
ErrorMessage: TypeAlias = str
Args: TypeAlias = Any
Kwargs: TypeAlias = Any

# Image Specific
Negative: TypeAlias = str
Width: TypeAlias = int
Height: TypeAlias = int
NoLogo: TypeAlias = bool
Enhance: TypeAlias = bool
Safe: TypeAlias = bool
Save: TypeAlias = bool


# Text Specific
System: TypeAlias = str
Contextual: TypeAlias = bool
Messages: TypeAlias = List[Dict[str, Any]]
Role: TypeAlias = Literal["system", "user", "assistant"]
Message: TypeAlias = Dict[str, Any]
JsonMode: TypeAlias = bool
ReasoningEffort: TypeAlias = Literal["low", "medium", "high"]
Tools: TypeAlias = List[Dict[str, Any]]
ToolChoice: TypeAlias = List[str]
Voice: TypeAlias = Literal["alloy", "echo", "fable", "onyx", "nova", "shimmer", "coral", "verse", "ballad", "ash", "sage", "amuch", "dan"]
UseOpenAIEndpoint: TypeAlias = bool
Images: TypeAlias = List[Dict[str, Any]]
ImageFormat: TypeAlias = dict[str, Union[str, dict[str, str]]]
Output: TypeAlias = Union[str, Any]


# Feed
FeedType: TypeAlias = Literal["image", "text"]
MaxData: TypeAlias = int


import pollinations

text_model = pollinations.Text(
    model="openai", 
    system="You are a helpful assistant.", 
    contextual=True,
    private=True,
    seed="random"
)

def main() -> None:
    while True:
        prompt = input("User: ")
        if prompt.lower().strip() == "exit":
            break
        
        print("Assistant: ", end="", flush=True)
        
        # print(text_model(prompt))
        for token in text_model(prompt, stream=True):
            print(token, end="", flush=True)

        print("\n")
        
main()


import pollinations

import asyncio

text_model = pollinations.Text()

async def main() -> None:
    async for token in await text_model.Async("This is a test.", stream=True):
        print(token, end="", flush=True)

    print()
    
asyncio.run(main())


from __future__ import annotations

from ..types import (
    StreamData,
    FeedType,
    MaxData,
    Infinity,
    Time,
    Args,
    Kwargs,
)

from .._core import (
    BaseClass,
    get_client,
    get_async_client,
    TEXT_API_URI,
    IMAGE_API_URI,
    _get_feed_request,
    _get_async_feed_request,
)

from typing import Self, Optional, List, Iterator, AsyncIterator
from time import time as now

__all__: list[str] = ["Feed"]


class Feed(BaseClass):
    def __init__(
        self: Self,
        type: FeedType = "image",
        max_data: Optional[MaxData | Infinity] = None,
        *args: Args,
        **kwargs: Kwargs,
    ) -> None:
        self._text_client = get_client(TEXT_API_URI + "feed")
        self._image_client = get_client(IMAGE_API_URI + "feed")
        self._async_text_client = get_async_client(TEXT_API_URI + "feed")
        self._async_image_client = get_async_client(IMAGE_API_URI + "feed")

        self.status = "DONE"

        self.type = type
        self.data: List[StreamData] = []
        self.max_data = max_data

    def __call__(
        self: Self, *args: Args, **kwargs: Kwargs
    ) -> Iterator[Feed.Data]:
        return self.Get(*args, **kwargs)

    def Get(self: Self, *args: Args, **kwargs: Kwargs) -> Iterator[Feed.Data]:
        self.status = "RUNNING"
        self.data.clear()
        counter = 0

        client = (
            self._image_client if self.type == "image" else self._text_client
        )
        for item in _get_feed_request(client, kwargs, *args, **kwargs):
            wrapped = self.Data(data=item, time=now())
            self.data.append(wrapped)
            yield wrapped
            counter += 1
            if self.max_data and counter >= self.max_data:
                break
            
        self.status = "DONE"

    async def Async(
        self: Self, *args: Args, **kwargs: Kwargs
    ) -> AsyncIterator[Feed.Data]:
        self.status = "RUNNING"
        self.data.clear()
        counter = 0

        client = (
            self._async_image_client
            if self.type == "image"
            else self._async_text_client
        )
        async for item in _get_async_feed_request(
            client, kwargs, *args, **kwargs
        ):
            wrapped = self.Data(data=item, time=now())
            self.data.append(wrapped)
            yield wrapped
            counter += 1
            if self.max_data and counter >= self.max_data:
                break

        self.status = "DONE"

    class Data(BaseClass):
        def __init__(
            self: Self,
            data: StreamData,
            time: Time,
            *args: Args,
            **kwargs: Kwargs,
        ) -> None:
            self.data = data
            self.time = time

        def __call__(self: Self, *args: Args, **kwargs: Kwargs) -> StreamData:
            return self.data

        def __getitem__(self: Self, key):
            return self.data[key]

        def __getattr__(self: Self, name):
            return getattr(self.data, name)

        def __iter__(self: Self):
            return iter(self.data)

        def __len__(self: Self):
            return len(self.data)

        def __contains__(self: Self, item):
            return item in self.data

        def keys(self: Self):
            return self.data.keys()

        def values(self: Self):
            return self.data.values()

        def items(self: Self):
            return self.data.items()




from ..types import Filename, Args, Kwargs
from ..errors import (
    EmptyFileError,
    FileNotFoundError,
    ImproperPermissionsError,
    NotAFileError,
)

import os

__all__: list[str] = ["_check_file"]


def _check_file(file: Filename, *args: Args, **kwargs: Kwargs) -> bool:
    if not os.path.exists(file):
        raise FileNotFoundError(f"{FileNotFoundError.DefaultMessage[:-1]} | {file}")
    if not os.path.isfile(file):
        raise NotAFileError(f"{NotAFileError.DefaultMessage[:-1]} | {file}")
    if not os.access(file, os.R_OK):
        raise ImproperPermissionsError(f"{ImproperPermissionsError.DefaultMessage[:-1]} | {file}")
    if os.path.getsize(file) == 0:
        raise EmptyFileError(f"{EmptyFileError.DefaultMessage[:-1]} | {file}")

    return True


# MIT License

# Copyright (c) 2025 pollinations

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .types import (
    PILImage,
    Prompt,
    Model,
    Seed,
    Private,
    File,
    Referrer,
    Stream,
    Filename,
    ImageData,
    Payload,
    Params,
    EmptyParams,
    StreamData,
    ErrorMessage,
    Args,
    Kwargs,
    Negative,
    Width,
    Height,
    NoLogo,
    Enhance,
    Safe,
    Save,
    System,
    Contextual,
    Messages,
    Role,
    Message,
    JsonMode,
    ReasoningEffort,
    Tools,
    ToolChoice,
    Voice,
    UseOpenAIEndpoint,
    Images,
    ImageFormat,
    Output,
    URL,
    Response,
    Request,
    Client,
    AsyncClient,
    TextModel,
    ImageModel
)
from .errors import (
    EmptyFileError,
    FailedToStreamError,
    FailedToTranscribeError,
    FileNotFoundError,
    ImproperPermissionsError,
    InvalidOrUnknownPromptError,
    NotAFileError
)
from ._core import (
    BaseClass,
    get_client,
    get_async_client,
)
from .core.feed import Feed
from .core.text import Text
from .core.image import Image
from .helpers import get_latest
from .version import (
    __title__,
    __version__,
    __license__,
    __description__
)


__all__ = [
    "PILImage", "Prompt", "Model", "Seed", "Private", "File", "Referrer", "Stream", "Filename",
    "ImageData", "Payload", "Params", "EmptyParams", "StreamData", "ErrorMessage", "Args", "Kwargs",
    "Negative", "Width", "Height", "NoLogo", "Enhance", "Safe", "Save", "System", "Contextual",
    "Messages", "Role", "Message", "JsonMode", "ReasoningEffort", "Tools", "ToolChoice", "Voice",
    "UseOpenAIEndpoint", "Images", "ImageFormat", "Output", "URL", "Response", "Request", "Client",
    "AsyncClient", "TextModel", "ImageModel",

    "EmptyFileError", "FailedToStreamError", "FailedToTranscribeError", "FileNotFoundError",
    "ImproperPermissionsError", "InvalidOrUnknownPromptError", "NotAFileError",

    "BaseClass", "get_client", "get_async_client", 
    
    "Feed", "Text", "Image",
    
    "get_latest",
    
    "__title__", "__version__", "__license__", "__description__"
]

__locals = locals()
for __name in __all__:
    if not __name.startswith("__"):
        try:
            __locals[__name].__module__ = "pollinations"
        except (TypeError, AttributeError):
            pass

get_latest()


from ..types import Role, Prompt, Output, ImageFormat, Message, Args, Kwargs

from typing import Optional, List, Union

__all__: list[str] = ["_create_message"]


def _create_message(role: Role, prompt: Prompt | Output, images: Optional[Union[ImageFormat, List[ImageFormat]]] = None, *args: Args, **kwargs: Kwargs) -> Message:
    message = {
        "role": role,
        "content": [{"type": "text", "text": prompt}]
    }
    
    if images:
        if isinstance(images, dict):
            images = [images]
        message["content"].extend(images)
        
    return message


from ..types import ErrorMessage

from typing import Self, LiteralString

__all__: list[str] = ["FileNotFoundError"]


class FileNotFoundError(Exception):
    DefaultMessage: LiteralString = "Could not find file."

    def __init__(self: Self, message: ErrorMessage) -> None:
        super().__init__(message)



from __future__ import annotations

from ..types import (
    TextModel,
    Model,
    System,
    Contextual,
    Messages,
    Private,
    Seed,
    ReasoningEffort,
    Tools,
    ToolChoice,
    Voice,
    JsonMode,
    Referrer,
    UseOpenAIEndpoint,
    Images,
    Prompt,
    Output,
    Request,
    Stream,
    StreamData,
    Params,
    Filename,
    PILImage,
    Args,
    Kwargs,
)
from .._utils.check_file import _check_file
from .._utils.format_image import _format_image
from .._utils.create_message import _create_message
from .._utils._models import get_text_models, get_async_text_models
from .._core import (
    BaseClass,
    get_client,
    get_async_client,
    TEXT_API_URI,
    _get_text_request,
    _get_async_text_request,
    _get_stream_text_request,
    _get_async_stream_text_request,
    _get_transcribe_request,
    _get_async_transcribe_request,
)
from ..errors import InvalidOrUnknownPromptError


from typing import Self, Optional, Union, List, Generator, AsyncGenerator


class Text(BaseClass):
    def __init__(
        self: Self,
        model: Optional[Model] = "openai",
        system: Optional[System] = "You are a helpful AI assistant.",
        contextual: Optional[Contextual] = False,
        messages: Optional[Messages] = [],
        private: Optional[Private] = False,
        seed: Optional[Seed] = "random",
        reasoning_effort: Optional[ReasoningEffort] = "medium",
        tools: Optional[Tools] = [],
        tool_choices: Optional[ToolChoice] = [],
        voice: Optional[Voice] = None,
        json_mode: Optional[JsonMode] = False,
        referrer: Optional[Referrer] = "pollinations.py",
        openai_endpoint: Optional[UseOpenAIEndpoint] = False,
        *any_kwargs_will_be_passed_in_request: Args,
        **kwargs: Kwargs,
    ) -> None:
        self._client = get_client(TEXT_API_URI)
        self._async_client = get_async_client(TEXT_API_URI)
        
        self.status = "DONE"

        self.model = model
        self.system = system
        self.contextual = contextual
        self.messages = messages
        self.private = private
        self.seed = seed
        self.reasoning_effort = reasoning_effort
        self.tools = tools
        self.tool_choices = tool_choices
        self.voice = voice
        self.json_mode = json_mode
        self.referrer = referrer
        self.openai_endpoint = openai_endpoint

        self.images: Images = None
        self.prompt: Prompt = None
        self.response: Output = None
        self.request: Request = None

        if self.system and not any(m.get("role") == "system" for m in self.messages):
            self.messages.insert(0, _create_message("system", self.system))


        for k, v in kwargs.items():
            setattr(self, k, v)

    def __call__(
        self: Self,
        prompt: Optional[Prompt] = None,
        *any_kwargs_will_be_passed_in_request: Args,
        stream: Optional[Stream] = False,
        **kwargs: Kwargs,
    ) -> Output:
        return self.Generate(
            prompt,
            *any_kwargs_will_be_passed_in_request,
            stream=stream,
            **kwargs,
        )

    def Generate(
        self: Self,
        prompt: Optional[Prompt] = None,
        *any_kwargs_will_be_passed_in_request: Args,
        stream: Optional[Stream] = False,
        **kwargs: Kwargs,
    ) -> Output:
        self.status = "RUNNING"
        prompt = self._check(prompt)

        if self.contextual:
            self.messages.append(_create_message("user", prompt))

        params = self._setup(prompt)


        if stream:

            def _generator() -> Generator[StreamData]:
                text = ""
                for chunk in _get_stream_text_request(
                    self._client, params, **kwargs
                ):
                    text += chunk
                    yield chunk
                self.response = text
                if self.contextual:
                    self.messages.append(
                        _create_message("assistant", self._decode(text))
                    )
                    
                self.status = "DONE"

            return _generator()
        else:
            self.request = _get_text_request(self._client, params, **kwargs)
            self.response = self._decode(self.request.content)
            if self.contextual:
                self.messages.append(
                    _create_message("assistant", self.response)
                )

            self.status = "DONE"
            return self.response

    async def Async(
        self: Self,
        prompt: Optional[Prompt] = None,
        *any_kwargs_will_be_passed_in_request: Args,
        stream: Optional[Stream] = False,
        **kwargs: Kwargs,
    ) -> Output:
        self.status = "RUNNING"
        prompt = self._check(prompt)

        if self.contextual:
            self.messages.append(_create_message("user", prompt))

        params = self._setup(prompt)

        if stream:

            async def _generator() -> AsyncGenerator[StreamData]:
                text = ""
                async for chunk in await _get_async_stream_text_request(
                    self._async_client, params, **kwargs
                ):
                    text += chunk
                    yield chunk
                self.response = text
                if self.contextual:
                    self.messages.append(
                        _create_message("assistant", self._decode(text))
                    )
                self.status = "DONE"

            return _generator()
        else:
            self.request = await _get_async_text_request(
                self._async_client, params, **kwargs
            )
            self.response = self._decode(self.request.content)
            if self.contextual:
                self.messages.append(
                    _create_message("assistant", self.response)
                )

            self.status = "DONE"
            return self.response

    def Transcribe(
        self: Self,
        file: Filename,
        *any_kwargs_will_be_passed_in_request: Args,
        **kwargs: Kwargs,
    ) -> Output:
        self.status = "RUNNING"
        params: Params = {"file": file, "seed": self.seed}
        request = _get_transcribe_request(
            self._client,
            params,
            *any_kwargs_will_be_passed_in_request,
            **kwargs,
        )
        self.status = "DONE"
        return request

    async def TranscribeAsync(
        self: Self,
        file: Filename,
        *any_kwargs_will_be_passed_in_request: Args,
        **kwargs: Kwargs,
    ) -> Output:
        self.status = "RUNNING"
        params: Params = {"file": file, "seed": self.seed}
        request = await _get_async_transcribe_request(
            self._async_client,
            params,
            *any_kwargs_will_be_passed_in_request,
            **kwargs,
        )
        self.status = "DONE"
        return request

    def Image(
        self: Self,
        file: Union[Filename, List[Filename]],
        *args: Args,
        **kwargs: Kwargs,
    ) -> PILImage:
        files = [file] if isinstance(file, Filename) else file
        self.images = [_format_image(f) for f in files if f and _check_file(f)]

    @staticmethod
    def Models() -> List[TextModel]:
        return get_text_models()

    @staticmethod
    async def ModelsAsync() -> List[TextModel]:
        return await get_async_text_models()

    def _check(self: Self, prompt: Optional[Prompt] = None) -> Prompt:
        if prompt:
            return prompt

        try:
            last = self.messages[-1]["content"]
            return last["text"] if isinstance(last, dict) else last
        except Exception:
            return InvalidOrUnknownPromptError(
                InvalidOrUnknownPromptError.DefaultMessage
            )

    def _setup(self: Self, prompt: Prompt) -> Params:
        params = {
            k: v
            for k, v in self.__dict__.items()
            if not k.startswith("_")
            and k not in {"images", "prompt", "response", "request", "system"}
        }

        if self.contextual:
            params["messages"] = self.messages
        else:
            params["messages"] = self.messages + [_create_message("user", prompt)]

        if self.system:
            params["__system"] = self.system
        if self.images:
            params["__images"] = self.images
        params["__prompt"] = prompt or " " 
        self.images = []
        return params


    @staticmethod
    def _decode(content: bytes) -> str:
        try:
            return content.decode("utf-8")
        except Exception:
            return str(content)



from .check_file import _check_file
from ..types import Filename, ImageData, Args, Kwargs

from io import BytesIO

__all__: list[str] = ["_read_image"]


def _read_image(file: Filename, *args: Args, **kwargs: Kwargs) -> ImageData:
    _check_file(file)

    with open(file, "rb") as f:
        return BytesIO(f.read())



from .read_image import _read_image
from ..types import Filename, ImageFormat, Args, Kwargs

from typing import Optional
from base64 import b64encode

__all__: list[str] = ["_format_image"]


def _format_image(
    file: Filename, *args: Args, **kwargs: Kwargs
) -> Optional[ImageFormat]:
    image_content = _read_image(file, *args, **kwargs)
    if image_content is None:
        return None

    encoded_image = b64encode(image_content.getvalue()).decode("utf-8")
    extensions = {
        "png": "image/png",
        "jpg": "image/jpeg",
        "jpeg": "image/jpeg",
        "gif": "image/gif",
        "webp": "image/webp",
    }

    extension = extensions.get(file.split(".")[-1].lower(), "image/jpeg")
    return {
        "type": "image_url",
        "image_url": {"url": f"data:{extension};base64,{encoded_image}"},
    }



import pollinations

import asyncio

feed = pollinations.Feed(
    type="image",
    max_data=10  # Optional: If None then will be indefinite
)

async def main() -> None:
    async for item in feed.Async(): 
        print(len(feed.data), item, item["width"])
        
asyncio.run(main())


from ..types import StreamData, Args, Kwargs

from json import loads, JSONDecodeError

from typing import Optional

__all__: list[str] = ["_stream_process", "_feed_stream_process"]


def _stream_process(line: str, *args, **kwargs) -> Optional[str]:
    line = line.strip()
    if not line:
        return ""

    if line in ("data: [DONE]", "[DONE]"):
        return None

    chunks = []
    if "data: " in line:
        parts = line.split("data: ")
        chunks = [
            f"data: {part}" if i > 0 else part
            for i, part in enumerate(parts)
            if part.strip()
        ]
    else:
        chunks = [line]

    all_content = []
    for chunk in chunks:
        chunk = chunk.strip()
        if not chunk:
            continue

        json_str = (
            chunk[len("data: ") :] if chunk.startswith("data: ") else chunk
        )

        try:
            data = loads(json_str)

            def _get(obj):
                if isinstance(obj, dict):
                    for key in ["content", "completion", "text", "response"]:
                        if (
                            key in obj
                            and isinstance(obj[key], str)
                            and obj[key]
                        ):
                            all_content.append(obj[key])
                            return

                    if "delta" in obj and isinstance(obj["delta"], dict):
                        delta = obj["delta"]
                        if (
                            "content" in delta
                            and isinstance(delta["content"], str)
                            and delta["content"]
                        ):
                            all_content.append(delta["content"])
                            return

                    for value in obj.values():
                        _get(value)

                elif isinstance(obj, list):
                    for item in obj:
                        _get(item)

            _get(data)

        except JSONDecodeError:
            if not chunk.startswith("data:") and not chunk.startswith("{"):
                all_content.append(chunk)

    return "".join(all_content)


def _feed_stream_process(
    line: StreamData, *args: Args, **kwargs: Kwargs
) -> StreamData:
    if line.startswith("data: "):
        data_str = line[len("data: ") :]
    else:
        data_str = line

    if data_str.strip() == "[DONE]":
        return None

    try:
        data = loads(data_str)
        return data
    except JSONDecodeError:
        return ""



from ..types import ErrorMessage

from typing import Self, LiteralString

__all__: list[str] = ["ImproperPermissionsError"]


class ImproperPermissionsError(Exception):
    DefaultMessage: LiteralString = "Attempted to access file with improper permissions."

    def __init__(self: Self, message: ErrorMessage) -> None:
        super().__init__(message)



from ..types import ErrorMessage

from typing import Self, LiteralString

__all__: list[str] = ["InvalidOrUnknownPromptError"]


class InvalidOrUnknownPromptError(Exception):
    DefaultMessage: LiteralString = "Prompt was invalid, none, or could not be found."

    def __init__(self: Self, message: ErrorMessage) -> None:
        super().__init__(message)



from .empty_file_error import EmptyFileError
from .failed_to_stream import FailedToStreamError
from .failed_to_transcribe import FailedToTranscribeError
from .file_not_found import FileNotFoundError
from .improper_permissions import ImproperPermissionsError
from .invalid_or_unkown_prompt import InvalidOrUnknownPromptError
from .not_a_file import NotAFileError

__all__: list[str] = [
    "EmptyFileError",
    "FailedToStreamError",
    "FailedToTranscribeError",
    "FileNotFoundError",
    "ImproperPermissionsError",
    "InvalidOrUnknownPromptError",
    "NotAFileError",
]


import pollinations

import pyaudio
import wave

# pollinations.Text().Transcribe("my_audio.mp3")

def main() -> None:
    p = pyaudio.PyAudio()
    chunk = 1024
    format = pyaudio.paInt16
    channels = 1
    rate = 44100
    duration = 5
    frames = []

    print("Speak")

    stream = p.open(
        format=format,
        channels=channels,
        rate=rate,
        input=True,
        frames_per_buffer=chunk,
    )

    for _ in range(0, int(rate / chunk * duration)):
        data = stream.read(chunk)
        frames.append(data)

    stream.stop_stream()
    stream.close()
    p.terminate()

    wf = wave.open("temp.wav", "wb")
    wf.setnchannels(channels)
    wf.setsampwidth(p.get_sample_size(format))
    wf.setframerate(rate)
    wf.writeframes(b"".join(frames))
    wf.close()

    text_model = pollinations.Text()
    result = text_model.Transcribe("temp.wav")
    print(result)

if __name__ == "__main__":
    main()



from ..types import Client, AsyncClient, TextModel, ImageModel, Args, Kwargs
from .._core import TEXT_API_URI, IMAGE_API_URI

from typing import List

__all__: list[str] = [
    "get_text_models",
    "get_image_models",
    "get_async_text_models",
    "get_async_image_models",
]


def _get_models(
    client: Client, *args: Args, **kwargs: Kwargs
) -> List[TextModel | ImageModel]:
    response = client.get("/models")
    response.raise_for_status()
    return response.json()


async def _get_async_models(
    async_client: AsyncClient, *args: Args, **kwargs: Kwargs
) -> List[TextModel | ImageModel]:
    response = await async_client.get(TEXT_API_URI)
    response.raise_for_status()
    return response.json()


def get_text_models(*args: Args, **kwargs: Kwargs) -> List[TextModel]:
    return _get_models(Client(base_url=TEXT_API_URI))


async def get_async_text_models(
    *args: Args, **kwargs: Kwargs
) -> List[TextModel]:
    return await _get_async_models(AsyncClient(base_url=TEXT_API_URI))


def get_image_models(*args: Args, **kwargs: Kwargs) -> List[ImageModel]:
    return _get_models(Client(base_url=IMAGE_API_URI))


async def get_async_image_models(
    *args: Args, **kwargs: Kwargs
) -> List[ImageModel]:
    return await _get_async_models(AsyncClient(base_url=IMAGE_API_URI))



from ..types import Prompt, System, Args, Kwargs

from urllib.parse import quote

__all__: list[str] = ["_encode_string"]


def _encode_string(string: Prompt | System, *args: Args, **kwargs: Kwargs) -> str:
    return quote(string)



from typing import Any, Self

__all__: list[str] = ["BaseClass"]


class BaseClass:
    def __repr__(self: Self) -> str:
        def fmt(val: Any) -> str:
            match val:
                case str() if len(val) > 30:
                    return repr(val[:27] + "...")
                case str():
                    return repr(val)
                case bytes() if len(val) > 30:
                    return repr(val[:27] + b"...")
                case list() if len(val) > 3:
                    return f"[{', '.join(fmt(v) for v in val[:3])}, ...]"
                case list():
                    return f"[{', '.join(fmt(v) for v in val)}]"
                case dict() if len(val) > 3:
                    items = ", ".join(
                        f"{fmt(k)}: {fmt(v)}" for k, v in list(val.items())[:3]
                    )
                    return f"{{{items}, ...}}"
                case dict():
                    items = ", ".join(
                        f"{fmt(k)}: {fmt(v)}" for k, v in val.items()
                    )
                    return f"{{{items}}}"
                case tuple() if len(val) > 3:
                    return f"({', '.join(fmt(v) for v in val[:3])}, ...)"
                case tuple():
                    return f"({', '.join(fmt(v) for v in val)})"
                case set() | frozenset() if len(val) > 3:
                    return f"{{{', '.join(fmt(v) for v in sorted(val)[:3])}, ...}}"
                case set() | frozenset():
                    return f"{{{', '.join(fmt(v) for v in sorted(val))}}}"
                case float():
                    return f"{val:.4g}"
                case int() | bool() | None:
                    return repr(val)
                case _:
                    return repr(val)

        public_attrs = {
            k: v for k, v in vars(self).items() if not k.startswith("_")
        }
        cls_name = self.__class__.__name__
        if not public_attrs:
            return f"{cls_name}()"
        body = ", ".join(
            f"{k}={fmt(v)}" for k, v in sorted(public_attrs.items())
        )
        return f"{cls_name}({body})"

    def __str__(self: Self) -> str:
        return self.__repr__()



from .._utils.check_file import _check_file
from .._utils._prepare_request import _prepare_request
from ..types import (
    Client,
    AsyncClient,
    Params,
    Payload,
    Request,
    Response,
    Args,
    Kwargs,
)
from ..errors import FailedToTranscribeError
from ._consts import (
    DEFAULT_HEADERS,
    DEFAULT_MAX_RETRIES,
    DEFAULT_TIMEOUT,
)

from base64 import b64encode

__all__: list[str] = [
    "_get_transcribe_request",
    "_get_async_transcribe_request",
]


def _get_transcribe_request(
    client: Client, params: Params, *args: Args, **kwargs: Kwargs
) -> Response:
    params = _prepare_request(params)
    file = params.pop("file")
    seed = params.pop("seed", 42)

    _check_file(file)

    with open(file, "rb") as f:
        data = b64encode(f.read()).decode("utf-8")        

    payload: Payload = {
        "model": "openai-audio",
        "messages": [
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": "Transcribe this audio. Do not wrap in quotes. Only raw text. Do not add anything else besides the transcription. Be as exact and precise as you can be.",
                    },
                    {
                        "type": "input_audio",
                        "input_audio": {
                            "data": data,
                            "format": file.split(".")[-1].lower(),
                        },
                    },
                ],
            }
        ],
        "seed": seed,
    }

    client.headers = DEFAULT_HEADERS
    request: Request = client.post(
        "openai", json=payload, *args, **kwargs, timeout=DEFAULT_TIMEOUT
    )
    request.raise_for_status()

    if request.status_code != 200:
        for _ in range(DEFAULT_MAX_RETRIES):
            request = client.post(
                "openai",
                json=payload,
                *args,
                **kwargs,
                timeout=DEFAULT_TIMEOUT,
            )
            if request.status_code == 200:
                break

    response = request.json()
    try:
        transcription = (
            response.get("choices", [{}])[0].get("message", {}).get("content")
        )
    except Exception:
        raise FailedToTranscribeError(
            f"{FailedToTranscribeError.DefaultMessage[:-1]} | {file}"
        )

    return transcription


async def _get_async_transcribe_request(
    client: AsyncClient, params: Params, *args: Args, **kwargs: Kwargs
) -> Response:
    params = _prepare_request(params)
    file = params.pop("file")
    seed = params.pop("seed", 42)

    _check_file(file)


    with open(file, "rb") as f:
        data = b64encode(f.read()).decode("utf-8")    

    payload: Payload = {
        "model": "openai-audio",
        "messages": [
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": "Transcribe this audio. Do not wrap in quotes. Only raw text. Do not add anything else besides the transcription. Be as exact and precise as you can be.",
                    },
                    {
                        "type": "input_audio",
                        "input_audio": {
                            "data": data,
                            "format": file.split(".")[-1].lower(),
                        },
                    },
                ],
            }
        ],
        "seed": seed,
    }

    client.headers = DEFAULT_HEADERS
    request: Request = await client.post(
        "openai", json=payload, *args, **kwargs, timeout=DEFAULT_TIMEOUT
    )
    request.raise_for_status()

    if request.status_code != 200:
        for _ in range(DEFAULT_MAX_RETRIES):
            request = await client.post(
                "openai",
                json=payload,
                *args,
                **kwargs,
                timeout=DEFAULT_TIMEOUT,
            )
            if request.status_code == 200:
                break

    response = request.json()
    try:
        transcription = (
            response.get("choices", [{}])[0].get("message", {}).get("content")
        )
    except Exception:
        raise FailedToTranscribeError(
            f"{FailedToTranscribeError.DefaultMessage[:-1]} | {file}"
        )
    return transcription



import pollinations

import random
import discord
from discord.ext import commands

intents = discord.Intents.all()
text_model = pollinations.Text()
image_model = pollinations.Image()

bot = commands.Bot(command_prefix="!", intents=intents)


@bot.command()
async def ask(ctx: commands.Context, *, prompt: str, **kwargs) -> None:
    message = await ctx.reply("Generating...")

    response = await text_model.Async(prompt)
    await message.edit(content=response)


@bot.command()
async def image(ctx: commands.Context, *, prompt: str, **kwargs) -> None:
    message = await ctx.reply("Generating...")

    image_model.file = f"{ctx.author.id}-{random.random()}.png"
    image = await image_model.Async(prompt)
    image.save(image_model.file)

    image = discord.File(image_model.file)
    await message.delete()
    await ctx.reply(content=f"`{prompt}`", file=image)


bot.run("TOKEN_HERE")



from .._utils._prepare_request import _prepare_request
from .._utils._clean_params import _clean_params
from ..types import (
    Client,
    AsyncClient,
    Params,
    Request,
    Response,
    Args,
    Kwargs,
)
from ._consts import (
    DEFAULT_HEADERS,
    DEFAULT_MAX_RETRIES,
    DEFAULT_TIMEOUT,
)

__all__: list[str] = ["_get_image_request", "_get_async_image_request"]


def _get_image_request(
    client: Client, params: Params, *args: Args, **kwargs: Kwargs
) -> Response:
    params = _prepare_request(params)
    prompt = params.pop("__iprompt")
    params = _clean_params(params)
    
    for k, v in params.items():
        if k == "__inegative":
            k = "negative"
        prompt += f"{k}={v}&"
    prompt = prompt[:-1]

    client.headers = DEFAULT_HEADERS
    request: Request = client.post(
        f"prompt/{prompt}", *args, **kwargs, timeout=DEFAULT_TIMEOUT
    )
    request.raise_for_status()

    if request.status_code != 200:
        for _ in range(DEFAULT_MAX_RETRIES):
            request = client.post(
                f"prompt/{prompt}", *args, **kwargs, timeout=DEFAULT_TIMEOUT
            )
            if request.status_code == 200:
                break
    return request


async def _get_async_image_request(
    client: AsyncClient, params: Params, *args: Args, **kwargs: Kwargs
) -> Response:
    params = _prepare_request(params)
    prompt = params.pop("__iprompt")
    params = _clean_params(params)
    
    for k, v in params.items():
        prompt += f"{k}={v}&"
    prompt = prompt[:-1]

    client.headers = DEFAULT_HEADERS
    request: Request = await client.post(
        f"prompt/{prompt}", *args, **kwargs, timeout=DEFAULT_TIMEOUT
    )
    request.raise_for_status()

    if request.status_code != 200:
        for _ in range(DEFAULT_MAX_RETRIES):
            request = await client.post(
                f"prompt/{prompt}", *args, **kwargs, timeout=DEFAULT_TIMEOUT
            )
            if request.status_code == 200:
                break
    return request



from typing import Final, Dict

from httpx import Timeout

__all__: list[str] = [
    "TEXT_API_URI",
    "IMAGE_API_URI",
    "DEFAULT_TIMEOUT",
    "DEFAULT_MAX_RETRIES",
    "DEFAULT_HEADERS",
]

TEXT_API_URI: Final[str] = "https://text.pollinations.ai/"
IMAGE_API_URI: Final[str] = "https://image.pollinations.ai/"

DEFAULT_TIMEOUT: Final[Timeout] = Timeout(timeout=100, 
                                          connect=5)
DEFAULT_MAX_RETRIES: Final[int] = 3

DEFAULT_HEADERS: Final[Dict[str, str]] = {
    "Accept": "application/json",
    "Content-Type": "application/json",
}



from ._base_class import BaseClass
from ._client import get_client, get_async_client
from ._consts import (
    TEXT_API_URI,
    IMAGE_API_URI,
    DEFAULT_TIMEOUT,
    DEFAULT_MAX_RETRIES,
    DEFAULT_HEADERS,
)
from ._get_feed_request import _get_feed_request, _get_async_feed_request
from ._get_image_request import _get_image_request, _get_async_image_request
from ._get_stream_request import _get_stream_text_request, _get_async_stream_text_request
from ._get_text_request import _get_text_request, _get_async_text_request
from ._get_transcribe_request import _get_transcribe_request, _get_async_transcribe_request

__all__: list[str] = [
    "BaseClass",
    "get_client",
    "get_async_client",
    "TEXT_API_URI",
    "IMAGE_API_URI",
    "DEFAULT_TIMEOUT",
    "DEFAULT_MAX_RETRIES",
    "DEFAULT_HEADERS",
    "_get_feed_request",
    "_get_async_feed_request",
    "_get_image_request",
    "_get_async_image_request",
    "_get_stream_text_request",
    "_get_async_stream_text_request",
    "_get_text_request",
    "_get_async_text_request",
    "_get_transcribe_request",
    "_get_async_transcribe_request",
]


from typing import TypedDict, Optional, List

__all__: list[str] = ["TextModel", "ImageModel"]

class TextModel(TypedDict, total=False):
    name: str
    description: str
    provider: str
    input_modalities: List[str]
    output_modalities: List[str]
    vision: bool
    audio: bool
    uncensored: Optional[bool]
    reasoning: Optional[bool]
    aliases: Optional[str]
    voices: Optional[List[str]]
    
class ImageModel(TypedDict, total=False):
    name: str


import pollinations

text_model = pollinations.Text()

def main() -> None:
    for token in text_model("This is a test.", stream=True):
        print(token, end="", flush=True)

    print()
    
main()


from ..types import Params, Args, Kwargs

__all__: list[str] = ["_clean_params"]


def _clean_params(params: Params, *args: Args, **kwargs: Kwargs) -> Params:
    for k in list(params.keys()):
        if k.startswith("_") or k == "openai_endpoint" or k == "status":
            params.pop(k)
            
    if "tools" in params and "tool_choices" in params:
        if params["tools"] == []:
            params.pop("tools")
        if params["tool_choices"] == []:
            params.pop("tool_choices")
            
    return params


from .._utils._prepare_request import _prepare_request
from .._utils._stream_process import _stream_process
from .._utils._clean_params import _clean_params
from ..types import (
    Client,
    AsyncClient,
    Params,
    StreamData,
    Args,
    Kwargs,
)
from ..errors import FailedToStreamError
from ._consts import (
    DEFAULT_HEADERS,
    DEFAULT_MAX_RETRIES,
    DEFAULT_TIMEOUT,
)

from httpx import HTTPError
from typing import Iterator, AsyncIterator, Optional, Callable

__all__: list[str] = [
    "_get_stream_text_request",
    "_get_async_stream_text_request",
]


def _get_stream_text_request(
    client: Client,
    params: Params,
    *args: Args,
    processor: Optional[
        Callable[[StreamData], Optional[StreamData]]
    ] = _stream_process,
    **kwargs: Kwargs
) -> Iterator[StreamData]:
    params = _prepare_request(params)
    params["stream"] = True
    _use_openai = params.pop("__openai", False)
    params = _clean_params(params)

    client.headers = DEFAULT_HEADERS

    for attempt in range(DEFAULT_MAX_RETRIES + 1):
        try:
            with client.stream(
                "POST",
                "" if _use_openai is False else "openai",
                json=params,
                *args,
                **kwargs,
                timeout=DEFAULT_TIMEOUT
            ) as response:
                response.raise_for_status()
                for line in response.iter_lines():
                    if not line:
                        continue
                    processed = processor(line)
                    if processed is None:
                        return
                    if processed:
                        yield processed
            return
        except HTTPError:
            if attempt == DEFAULT_MAX_RETRIES:
                raise FailedToStreamError(FailedToStreamError.DefaultMessage)


async def _get_async_stream_text_request(
    client: AsyncClient,
    params: Params,
    *args: Args,
    processor: Optional[
        Callable[[StreamData], Optional[StreamData]]
    ] = _stream_process,
    **kwargs: Kwargs
) -> AsyncIterator[StreamData]:
    params = _prepare_request(params)
    params["stream"] = True
    _use_openai = params.pop("__openai", False)
    params = _clean_params(params)

    client.headers = DEFAULT_HEADERS

    for attempt in range(DEFAULT_MAX_RETRIES + 1):
        try:

            async def _generator() -> AsyncIterator[StreamData]:
                async with client.stream(
                    "POST",
                    "" if _use_openai is False else "openai",
                    json=params,
                    *args,
                    **kwargs,
                    timeout=DEFAULT_TIMEOUT
                ) as response:
                    response.raise_for_status()
                    async for line in response.aiter_lines():
                        if not line:
                            continue
                        processed = processor(line)
                        if processed is None:
                            return
                        if processed:
                            yield processed
                return

            return _generator()

        except HTTPError:
            if attempt == DEFAULT_MAX_RETRIES:
                raise FailedToStreamError(FailedToStreamError.DefaultMessage)



from ._encode_string import _encode_string
from ..types import Params, EmptyParams, Args, Kwargs

from random import randint

__all__: list[str] = ["_prepare_request"]


def _prepare_request(params: Params, *args: Args, **kwargs: Kwargs) -> Params | EmptyParams:
    if params is None:
        params = {}

    params["__iprompt"] = _encode_string((params.pop("__iprompt", "Error symbol alert, simple vector grapic.") + "?"))
    params["__inegative"] = _encode_string(params.pop("__inegative", "Detailed, lots of detail"))
    
    params["json"] = params.pop("json_mode", False)
    prompt = _encode_string(params.pop("__prompt", ""))
    system = _encode_string(params.pop("__system", ""))
    images = params.pop("__images", None)
    seed = params.pop("seed", None)
    

    if images or "messages" not in params or not params["messages"]:
        messages = []

        if system:
            messages.append({"role": "system", "content": system})

        if prompt:
            content = [{"type": "text", "text": prompt}]
            if images:
                if isinstance(images, dict):
                    content.append(images)
                else:
                    content.extend(images)
            messages.append({"role": "user", "content": content})

        params["messages"] = messages

    if seed:
        params["seed"] = (
            seed
            if isinstance(seed, (int, float))
            else randint(-2147483648, 2147483647)
        )
        
    return params



from .core import Text, Image
from .version import __version__

import argparse
from typing import List 

__all__: List[str] = ["main", "Text", "Image"]

def main() -> None:
    parser = argparse.ArgumentParser(
        prog="pollinations",
        description="Work with the best generative AI from Pollinations using this Python SDK. 🐝"
    )

    parser.add_argument(
        "--version", action="version", version="%(prog)s " + __version__
    )

    parser.add_argument("--text", action="store_true", help="Generate text")
    parser.add_argument("--image", action="store_true", help="Generate images")

    parser.add_argument("--prompt", type=str, help="Prompt to generate text or image")
    parser.add_argument(
        "--negative", type=str, help="Negative prompt to generate images", default=""
    )
    parser.add_argument("--model", type=str, help="Model to use", default="flux")
    parser.add_argument(
        "--system",
        type=str,
        help="System prompt for text models",
        default="You are a helpful AI assistant.",
    )
    parser.add_argument(
        "--file",
        type=str,
        help="The file name to save generated images",
        default="pollinations-image.png",
    )
    parser.add_argument(
        "--private",
        type=str,
        help="Private request (not displayed on public feed for text and images) | USE: (t, f, true, false) Case insensitive",
        default=False,
    )
    parser.add_argument(
        "--seed", type=str, help="Use: int or `random`", default="random"
    )
    parser.add_argument(
        "--width", type=int, help="The width for generated images", default=1024
    )
    parser.add_argument(
        "--height", type=int, help="The height for generated images", default=1024
    )
    parser.add_argument(
        "--nologo", type=bool, help="Remove the pollinations watermark", default=False
    )
    parser.add_argument(
        "--enhance",
        type=bool,
        help="Use AI to enhance your image prompts",
        default=False,
    )
    
    args = parser.parse_args()
    first_sent = False
    
    if args.text:
        text_model = Text(
            model=args.model, system=args.system, private=args.private, seed=args.seed
        )

        print("Pollinations - Waiting...")
        for token in text_model(args.prompt, stream=True):
            if not first_sent:
                print("\033[F\033[K", end="")
            first_sent = True
            print(token, end="", flush=True)

        print()
        first_sent = False

    if args.image:
        image_model = Image(
            model=args.model,
            seed=args.seed,
            width=args.width,
            height=args.height,
            private=args.private,
            enhance=args.enhance,
        )

        print("Pollinations - Waiting...")
        image_model(args.prompt, negative=args.negative, file=args.file, save=True)
        print("\033[F\033[K", end="")
        first_sent = False


from __future__ import annotations

from ..types import (
    ImageModel,
    Model,
    Private,
    Negative,
    Width,
    Height,
    NoLogo,
    Enhance,
    Safe,
    Save,
    File,
    Seed,
    Referrer,
    Prompt,
    Output,
    Request,
    Params,
    Filename,
    PILImage,
    Args,
    Kwargs,
)
from .._utils.to_pil import _to_pil
from .._utils._models import get_image_models, get_async_image_models
from .._core import (
    BaseClass,
    get_client,
    get_async_client,
    IMAGE_API_URI,
    _get_image_request,
    _get_async_image_request
)

from typing import Self, Optional, List


class Image(BaseClass):
    def __init__(
        self: Self,
        model: Optional[ImageModel] = "flux",
        width: Optional[Width] = 1024,
        height: Optional[Height] = 1024,
        seed: Optional[Seed] = "random",
        nologo: Optional[NoLogo] = False,
        private: Optional[Private] = False,
        enhance: Optional[Enhance] = False,
        safe: Optional[Safe] = False,
        referrer: Optional[Referrer] = "pollinations.py",
        *any_kwargs_will_be_passed_in_request: Args,
        **kwargs: Kwargs,
    ) -> None:
        self._client = get_client(IMAGE_API_URI)
        self._async_client = get_async_client(IMAGE_API_URI)
        
        self.status = "DONE"

        self.model = model
        self.width = width
        self.height = height
        self.seed = seed
        self.nologo = nologo
        self.private = private
        self.enhance = enhance
        self.safe = safe
        self.referrer = referrer
        
        self.prompt: Prompt = None
        self.negative: Negative = None
        self.file: File = "pollinations-image.jpeg"
        
        self.image: PILImage = None
        self.response: Output = None
        self.request: Request = None

        for k, v in kwargs.items():
            setattr(self, k, v)
            
    def __call__(self: Self, prompt: Prompt, negative: Optional[Negative] = "", *args: Args, file: Optional[Filename] = "pollinations-image.jpeg", save: Save = False, **kwargs: Kwargs) -> PILImage:
        return self.Generate(prompt, negative, *args, file=file, save=save, **kwargs)
    
    def Generate(self: Self, prompt: Prompt, negative: Optional[Negative] = "", *args: Args, file: Optional[Filename] = "pollinations-image.jpeg", save: Save = False, **kwargs: Kwargs) -> PILImage:
        self.status = "RUNNING"
        self.prompt = prompt
        self.negative = negative
        self.file = file
        
        params = self._setup()
        params["__iprompt"] = prompt
        params["__inegative"] = negative
        
        self.request = _get_image_request(self._client, params=params)
        self.response = self.request.content
        self.image = _to_pil(self.response)
        
        if save:
            self.image.save(self.file)
            
        self.status = "DONE"
        
        return self.image
    
    async def Async(self: Self, prompt: Prompt, negative: Optional[Negative] = "", *args: Args, file: Optional[Filename] = "pollinations-image.jpeg", save: Save = False, **kwargs: Kwargs) -> PILImage:
        self.status = "RUNNING"
        self.prompt = prompt
        self.negative = negative
        self.file = file
        
        params = self._setup()
        params["__iprompt"] = prompt
        params["__inegative"] = negative
        
        self.request = await _get_async_image_request(self._async_client, params=params)
        self.response = self.request.content
        self.image = _to_pil(self.response)
        
        if save:
            self.image.save(self.file)
            
        self.status = "DONE"
        
        return self.image
    
    @staticmethod
    def Models() -> List[Model]:
        return get_image_models()
    
    @staticmethod
    async def ModelsAsync() -> List[Model]:
        return await get_async_image_models()
    
    def _setup(self: Self) -> Params:
        params: Params = {
            k: v
            for k, v in self.__dict__.items()
            if not k.startswith("_")
            and k not in {"prompt", "negative", "image", "response", "request", "file"}
        }
        
        return params


from .._utils._prepare_request import _prepare_request
from .._utils._stream_process import _feed_stream_process
from .._utils._clean_params import _clean_params
from ..types import (
    Client,
    AsyncClient,
    Params,
    StreamData,
    Args,
    Kwargs,
)
from ..errors import FailedToStreamError
from ._consts import (
    DEFAULT_HEADERS,
    DEFAULT_MAX_RETRIES,
    DEFAULT_TIMEOUT,
)

from httpx import HTTPError, HTTPStatusError
from typing import Iterator, AsyncIterator, Optional, Callable

__all__: list[str] = [
    "_get_feed_request",
    "_get_async_feed_request",
]


def _get_feed_request(
    client: Client,
    params: Params,
    *args: Args,
    processor: Optional[Callable[[StreamData], Optional[StreamData]]] = _feed_stream_process,
    **kwargs: Kwargs
) -> Iterator[StreamData]:
    params = _prepare_request(params)
    params = _clean_params(params)
    client.headers = DEFAULT_HEADERS

    for attempt in range(DEFAULT_MAX_RETRIES + 1):
        for method in ("POST", "GET"):  
            try:
                stream_args = {
                    "method": method,
                    "url": "",
                    "timeout": DEFAULT_TIMEOUT,
                    **kwargs
                }
                if method == "POST":
                    stream_args["json"] = params

                with client.stream(**stream_args) as response:
                    response.raise_for_status()
                    for line in response.iter_lines():
                        if not line:
                            continue
                        processed = processor(line)
                        if processed is None:
                            return
                        if processed:
                            yield processed
                return
            except HTTPStatusError as e:
                if e.response.status_code == 404 and method == "POST":
                    continue 
                if attempt == DEFAULT_MAX_RETRIES:
                    raise FailedToStreamError(FailedToStreamError.DefaultMessage)
            except HTTPError:
                if attempt == DEFAULT_MAX_RETRIES:
                    raise FailedToStreamError(FailedToStreamError.DefaultMessage)


async def _get_async_feed_request(
    client: AsyncClient,
    params: Params,
    *args: Args,
    processor: Optional[Callable[[StreamData], Optional[StreamData]]] = _feed_stream_process,
    **kwargs: Kwargs
) -> AsyncIterator[StreamData]:
    params = _prepare_request(params)
    params = _clean_params(params)
    client.headers = DEFAULT_HEADERS

    for attempt in range(DEFAULT_MAX_RETRIES + 1):
        for method in ("POST", "GET"):
            try:
                stream_args = {
                    "method": method,
                    "url": "",
                    "timeout": DEFAULT_TIMEOUT,
                    **kwargs
                }
                if method == "POST":
                    stream_args["json"] = params

                async with client.stream(**stream_args) as response:
                    response.raise_for_status()
                    async for line in response.aiter_lines():
                        if not line:
                            continue
                        processed = processor(line)
                        if processed is None:
                            return
                        if processed:
                            yield processed
                return
            except HTTPStatusError as e:
                if e.response.status_code == 404 and method == "POST":
                    continue
                if attempt == DEFAULT_MAX_RETRIES:
                    raise FailedToStreamError(FailedToStreamError.DefaultMessage)
            except HTTPError:
                if attempt == DEFAULT_MAX_RETRIES:
                    raise FailedToStreamError(FailedToStreamError.DefaultMessage)


from .aliases import (
    PILImage,
    Prompt,
    Model,
    Seed,
    Private,
    File,
    Referrer,
    Stream,
    Filename,
    ImageData,
    Infinity,
    Time,
    Payload,
    Params,
    EmptyParams,
    StreamData,
    ErrorMessage,
    Args,
    Kwargs,
    Negative,
    Width,
    Height,
    NoLogo,
    Enhance,
    Safe,
    Save,
    System,
    Contextual,
    Messages,
    Role,
    Message,
    JsonMode,
    ReasoningEffort,
    Tools,
    ToolChoice,
    Voice,
    UseOpenAIEndpoint,
    Images,
    ImageFormat,
    Output,
    FeedType,
    MaxData
)
from .httpx import URL, Response, Request, Client, AsyncClient
from .model import TextModel, ImageModel

__all__ = [
    "PILImage",
    "Prompt",
    "Model",
    "Seed",
    "Private",
    "File",
    "Referrer",
    "Stream",
    "Filename",
    "ImageData",
    "Infinity",
    "Time",
    "Payload",
    "Params",
    "EmptyParams",
    "StreamData",
    "ErrorMessage",
    "Args",
    "Kwargs",
    "Negative",
    "Width",
    "Height",
    "NoLogo",
    "Enhance",
    "Safe",
    "Save",
    "System",
    "Contextual",
    "Messages",
    "Role",
    "Message",
    "JsonMode",
    "ReasoningEffort",
    "Tools",
    "ToolChoice",
    "Voice",
    "UseOpenAIEndpoint",
    "Images",
    "ImageFormat",
    "Output",
    "FeedType",
    "MaxData",
    "URL",
    "Response",
    "Request",
    "Client",
    "AsyncClient",
    "TextModel",
    "ImageModel",
]



import pollinations

feed = pollinations.Feed(
    type="image",
    max_data=10  # Optional: If None then will be indefinite
)

def main() -> None:
    for item in feed():  # feed.Get()
        print(len(feed.data), item, item["width"])
        
main()


from ..types import URL, Client, AsyncClient, Args, Kwargs

__all__: list[str] = ["get_client", "get_async_client"]

def get_client(base_url: URL, *args: Args, **kwargs: Kwargs) -> Client:
    return Client(base_url=base_url)

def get_async_client(base_url: URL, *args: Args, **kwargs: Kwargs) -> AsyncClient:
    return AsyncClient(base_url=base_url)


from ..types import ImageData, PILImage, Args, Kwargs

from io import BytesIO
from PIL import Image

__all__: list[str] = ["_to_pil"]

def _to_pil(data: ImageData, *args: Args, **kwargs: Kwargs) -> PILImage:
    return Image.open(data if isinstance(data, BytesIO) else BytesIO(data))



import json
import urllib.request
from importlib.metadata import version as get_installed_version
from packaging.version import Version, parse as parse_version

__all__: list[str] = ["get_latest"]

def get_latest() -> None:
    name = "pollinations"
    installed: Version = parse_version(get_installed_version(name))

    with urllib.request.urlopen(f"https://pypi.org/pypi/{name}/json", timeout=3) as res:
        data = json.load(res)
        latest: Version = parse_version(data["info"]["version"])

    if (installed.major, installed.minor) != (latest.major, latest.minor):
        raise RuntimeError(
            f"Pollinations Version {installed} is outdated. "
            f"Latest is {latest}. Update with:\n\n    pip install -U {name}"
        )
